#include <iostream>

#include "qintdict.h"
#include "qsettings.h"

#include "config.h"

using namespace std;

int activeprofile;
Config *global_config;

QIntDict<Config> profiles;
QIntDict<DcHubConfig> hubconfigs;

#define confprefix QString("/freed/cdget")

/* QSettings object used to write cache download updates */
static SettingsWriter sw;

/* Create a default configuration */
void configSetup()
{
  profiles.setAutoDelete(true);
  hubconfigs.setAutoDelete(true);
  if (!readConfig()) {
    cerr << "Unable to read config, reverting to default.\n";
    setActiveConfig(newConfig());
  }
}

bool setActiveConfig(int id)
{
  Config *tmp = profiles.find(id);
  if (tmp) {
    global_config = tmp;
    activeprofile = id;
    return true;
  }
  return false;
}

int getActiveConfig()
{
  return activeprofile;
}

bool readConfig()
{
  QSettings s;
  s.beginGroup(confprefix);
  hubconfigs.clear();
  s.beginGroup("/hubs");
  QStringList hubs = s.subkeyList("/");
  QStringList::iterator hit;

  for ( hit = hubs.begin(); hit != hubs.end(); ++hit) {
    QString curhub = (*hit);
    s.beginGroup("/"+curhub);
    int hid = s.readNumEntry("/id",0);
    if (profiles.find(hid)) {
      profiles.clear();
      cerr << "readConfig(): Multiple hubs with same id.\n";
      return false;
    }
    
    DcHubConfig *hconf = new DcHubConfig();
    
    hconf->name        = s.readEntry("/name","");
    hconf->hubaddr     = s.readEntry("/hubaddr","");
    hconf->nick        = s.readEntry("/nick","cdget");
    hconf->desc        = s.readEntry("/desc","");
    hconf->email       = s.readEntry("/email","");
    hconf->conn        = s.readEntry("/conn","Cable");
    hconf->shareddirs  = s.readListEntry("/shareddirs");
    hconf->usesizeoffs = s.readBoolEntry("/usesizeoffs",false);
    hconf->sizeoffs    = s.readDoubleEntry("/sizeoffs",0);
    hconf->usevshare   = s.readBoolEntry("/usevshare",false);
    hconf->vsharedir   = s.readEntry("/vsharedir","");
    hconf->maxupl      = s.readNumEntry("/maxupl",3);
    hconf->useullim    = s.readBoolEntry("/useullim",false);
    hconf->ullim       = s.readNumEntry("/ullim",0);
    
    hubconfigs.insert(hid,hconf);
    s.endGroup();
  }
  s.endGroup();
  s.beginGroup("/profiles");

  QStringList profs = s.subkeyList("/");

  if (profs.empty()) {
    hubconfigs.clear();
    cerr << "readConfig(): No profiles!\n";
    return false;
  }

  profiles.clear();
  QStringList::iterator it;    
  for ( it = profs.begin(); it != profs.end(); ++it ) {
    QString curprofile = (*it);
    s.beginGroup("/"+curprofile);
    int pid = s.readNumEntry("/id",0);
    if (profiles.find(pid)) {
      hubconfigs.clear();
      profiles.clear();
      cerr << "readConfig(): Multiple profiles with same id.\n";
      return false;
    }

    Config *conf = new Config();
    conf->name = s.readEntry("/name","default");
    
    conf->downloaddir = s.readEntry("/downloaddir",".");
    conf->fileformat  = s.readEntry("/fileformat","%a/%a - %l/%a - %n - %t.mp3");
    conf->useid3v2    = s.readBoolEntry("/useid3v2",false);
    conf->setid3      = s.readBoolEntry("/setid3",false);
    conf->rejectid3   = s.readBoolEntry("/rejectid3",false);
    conf->tempdir     = s.readEntry("/tempdir",".");
    conf->tempprefix  = s.readEntry("/tempprefix","CdGet-Downloadfile");
    conf->viewdetails = s.readBoolEntry("/viewdetails",false);

    conf->reqartist   = s.readBoolEntry("/reqartist",true);
    conf->reqalbum    = s.readBoolEntry("/reqalbum",false);
    conf->matchchars  = s.readEntry("/matchchars","a-zA-Z'");
    conf->minbps      = s.readNumEntry("/minbps",8192);
    conf->maxbps      = s.readNumEntry("/maxbps",32768);
    
    conf->delay              = s.readNumEntry("/delay",5);
    conf->dlcand             = s.readNumEntry("/dlcand",3);
    conf->canddelay          = s.readNumEntry("/canddelay",10);
    conf->candtimeout        = s.readNumEntry("/candtimeout",15);
    conf->candtryout         = s.readNumEntry("/candtryout",15);
    conf->maxqueue           = s.readNumEntry("/maxqueue",20);
    conf->maxgdlqueue        = s.readNumEntry("/maxgdlqueue",5);
    conf->nosourcetimeout    = s.readNumEntry("/nosourcetimeout",30);
    conf->stalledtimeout     = s.readNumEntry("/stalledtimeout",60);
    conf->searchtimeout      = s.readNumEntry("/searchtimeout",90);
    conf->gdlnosourcetimeout = s.readNumEntry("/gdlnosourcetimeout",20);
    conf->gdlstalltimeout    = s.readNumEntry("/gdlstalltimeout",30);
    conf->gdlsourcestalltimeout = s.readNumEntry("/gdlsourcestalltimeout",30);
    conf->gdlsourceattempts  = s.readNumEntry("/gdlsourceattempts",3);

    /* Connection options */
    conf->activemode = s.readBoolEntry("/activemode",true);

    conf->useoverrideip   = s.readBoolEntry("/useoverrideip",false);
    conf->overrideip      = s.readEntry("/overideip","");
    conf->useoverrideport = s.readBoolEntry("/useoverrideport",false);
    conf->overrideport    = s.readNumEntry("/overrideport",412);
    conf->dynamicip       = s.readBoolEntry("/dynamicip",false);

    conf->useproxy      = s.readBoolEntry("/useproxy",false);
    conf->proxyversion  = static_cast<ProxyVerEnum>(s.readNumEntry("/proxyversion",SOCKS4));
    conf->proxyaddr     = s.readEntry("/proxyaddr","");
    conf->proxyport     = s.readNumEntry("/proxyport",1080);
    conf->proxyuser     = s.readEntry("/proxyuser","");
    conf->proxypass     = s.readEntry("/proxypass","");

    conf->hubsproxytype = static_cast<HubsProxyEnum>(s.readNumEntry("/hubsproxytype",NoProxy));
    conf->hubsproxyaddr = s.readEntry("/hubsproxyaddr","");
    conf->hubsproxyport = s.readNumEntry("/hubsproxyport",80);

  /* Active hubs */
    QStringList activehubsstr = s.readListEntry("/activehubs");
    /* Check that these are valid and convert to integers */
    QStringList::iterator ait;
    for (ait = activehubsstr.begin(); ait != activehubsstr.end(); ++ait) {
      QString curhub = (*ait);
      bool cok;
      int id = curhub.toInt(&cok);
      if (!cok || !hubconfigs.find(id)) {
	hubconfigs.clear();
	profiles.clear();
	delete conf;
	cerr << "readConfig(): Invalid hub in active hubs list.\n";
	return false;
      }
      conf->activehubs.push_back(id);
    }

    profiles.insert(pid,conf);
    s.endGroup();
  }
  s.endGroup();

  int activeid = s.readNumEntry("/activeprofile",0);
  if (!setActiveConfig(activeid)) {
    hubconfigs.clear();
    profiles.clear();
    cerr << "readConfig(): Active profile refering to non-existing profile.\n";
    return false;
  }
  s.endGroup();
  return true;
}

void writeConfig()
{
  QSettings s;
  s.beginGroup(confprefix);

  s.beginGroup("/hubs");
  /* Remove all old hubs */
  QStringList hubs = s.subkeyList("/");
  QStringList::iterator hrit;    
  for ( hrit = hubs.begin(); hrit != hubs.end(); ++hrit ) {
    s.beginGroup("/"+*hrit);
    QStringList items = s.entryList("/");
    QStringList::iterator iit;
    for ( iit = items.begin(); iit != items.end(); ++iit) {
      s.removeEntry("/"+*iit);
    }
    s.endGroup();
  }

  QIntDictIterator<DcHubConfig> hit(hubconfigs);
  DcHubConfig *curhconf;
  while ((curhconf = hit.current())) {
    int hubid = hit.currentKey();
    QString hubname;
    hubname.setNum(hubid);
    hubname = "hub-"+hubname;
    s.beginGroup("/"+hubname);

    s.writeEntry("/id",hubid);
    s.writeEntry("/name",curhconf->name);
    s.writeEntry("/hubaddr",curhconf->hubaddr);
    s.writeEntry("/nick",curhconf->nick);
    s.writeEntry("/desc",curhconf->desc);
    s.writeEntry("/email",curhconf->email);
    s.writeEntry("/conn",curhconf->conn);
    s.writeEntry("/shareddirs",curhconf->shareddirs);
    s.writeEntry("/usesizeoffs",curhconf->usesizeoffs);
    s.writeEntry("/sizeoffs",(double)curhconf->sizeoffs);
    s.writeEntry("/usevshare",curhconf->usevshare);
    s.writeEntry("/vsharedir",curhconf->vsharedir);
    s.writeEntry("/maxupl",curhconf->maxupl);
    s.writeEntry("/useullim",curhconf->useullim);
    s.writeEntry("/ullim",curhconf->ullim);
    
    s.endGroup();
    ++hit;
  }
  s.endGroup();
  s.beginGroup("/profiles");

  /* Remove all old profiles */
  QStringList profs = s.subkeyList("/");
  QStringList::iterator rit;
  for ( rit = profs.begin(); rit != profs.end(); ++rit ) {
    s.beginGroup("/"+*rit);
    QStringList items = s.entryList("/");
    QStringList::iterator iit;
    for ( iit = items.begin(); iit != items.end(); ++iit) {
      s.removeEntry("/"+*iit);
    }
    s.endGroup();
  }

  QIntDictIterator<Config> it(profiles);
  Config *curconf;
  while ((curconf = it.current())) {
    int profid = it.currentKey();
    QString profname;
    profname.setNum(profid);
    profname = "profile-"+profname;
    s.beginGroup("/"+profname);

    s.writeEntry("/id",profid);
    s.writeEntry("/name",curconf->name);
      
    s.writeEntry("/downloaddir",curconf->downloaddir);
    s.writeEntry("/fileformat",curconf->fileformat);
    s.writeEntry("/useid3v2",curconf->useid3v2);
    s.writeEntry("/setid3",curconf->setid3);
    s.writeEntry("/rejectid3",curconf->rejectid3);
    s.writeEntry("/tempdir",curconf->tempdir);
    s.writeEntry("/tempprefix",curconf->tempprefix);
    s.writeEntry("/viewdetails",curconf->viewdetails);
      
    s.writeEntry("/reqartist",curconf->reqartist);
    s.writeEntry("/reqalbum",curconf->reqalbum);
    s.writeEntry("/matchchars",curconf->matchchars);
    s.writeEntry("/minbps",curconf->minbps);
    s.writeEntry("/maxbps",curconf->maxbps);
      
    s.writeEntry("/delay",curconf->delay);
    s.writeEntry("/dlcand",curconf->dlcand);
    s.writeEntry("/canddelay",curconf->canddelay);
    s.writeEntry("/candtimeout",curconf->candtimeout);
    s.writeEntry("/candtryout",curconf->candtryout);
    s.writeEntry("/maxqueue",curconf->maxqueue);
    s.writeEntry("/maxgdlqueue",curconf->maxgdlqueue);
    s.writeEntry("/nosourcetimeout",curconf->nosourcetimeout);
    s.writeEntry("/stalledtimeout",curconf->stalledtimeout);
    s.writeEntry("/searchtimeout",curconf->searchtimeout);
    s.writeEntry("/gdlnosourcetimeout",curconf->gdlnosourcetimeout);
    s.writeEntry("/gdlstalltimeout",curconf->gdlstalltimeout);
    s.writeEntry("/gdlsourcestalltimeout",curconf->gdlsourcestalltimeout);
    s.writeEntry("/gdlsourceattempts",curconf->gdlsourceattempts);

    /* Connection settings */
    s.writeEntry("/activemode",curconf->activemode);
    
    s.writeEntry("/useoverrideip",curconf->useoverrideip);
    s.writeEntry("/overideip",curconf->overrideip);
    s.writeEntry("/useoverrideport",curconf->useoverrideport);
    s.writeEntry("/overrideport",curconf->overrideport);
    s.writeEntry("/dynamicip",curconf->dynamicip);
    
    s.writeEntry("/useproxy",curconf->useproxy);
    s.writeEntry("/proxyversion",curconf->proxyversion);
    s.writeEntry("/proxyaddr",curconf->proxyaddr);
    s.writeEntry("/proxyport",curconf->proxyport);
    s.writeEntry("/proxyuser",curconf->proxyuser);
    s.writeEntry("/proxypass",curconf->proxypass);
    
    s.writeEntry("/hubsproxytype",curconf->hubsproxytype);
    s.writeEntry("/hubsproxyaddr",curconf->hubsproxyaddr);
    s.writeEntry("/hubsproxyport",curconf->hubsproxyport);

    /* Writing active hubs require conversion to strings */
    QStringList hubs;
    QValueList<int>::iterator tit;
    for (tit = curconf->activehubs.begin(); tit != curconf->activehubs.end(); ++tit) {
      int hubid = *tit;
      QString hubidstr;
      hubidstr.setNum(hubid);
      hubs.push_back(hubidstr);
    }
   
    s.writeEntry("/activehubs",hubs);
	
     s.endGroup();
    ++it;
  }
  s.endGroup();
  s.writeEntry("/activeprofile",activeprofile);
  s.endGroup();
}

int newConfig()
{
  /* We didn't have a saved profile, make one new up */
  Config *newconf = new Config();
  
  newconf->name = "Default";
  newconf->downloaddir = ".";
  newconf->fileformat = "%a/%a - %l/%a - %n - %t.mp3";
  newconf->useid3v2 = false;
  newconf->setid3 = false;
  newconf->rejectid3 = false;
  newconf->tempdir = ".";
  newconf->tempprefix = "CdGet-Temporary-";
  newconf->viewdetails = false;
  
  newconf->reqartist = true;
  newconf->reqalbum  = false;
  newconf->matchchars = "a-zA-Z'";
  newconf->minbps = 8192;
  newconf->maxbps = 32768;
  
  newconf->delay = 5;
  newconf->dlcand = 3;
  newconf->canddelay = 10;
  newconf->candtimeout = 15;
  newconf->candtryout  = 15;
  newconf->maxqueue    = 20;
  newconf->maxgdlqueue = 5;
  newconf->nosourcetimeout = 30;
  newconf->stalledtimeout = 60;
  newconf->searchtimeout = 90;
  newconf->gdlnosourcetimeout = 20;
  newconf->gdlstalltimeout = 30;
  newconf->gdlsourcestalltimeout = 30;
  newconf->gdlsourceattempts = 3;
  // newconf->activehubs; No active hubs as default?

  /* Connection options */
  newconf->activemode = true;

  newconf->useoverrideip = false;
  newconf->overrideip = "";
  newconf->useoverrideport = false;
  newconf->overrideport = 412;
  newconf->dynamicip = false;
  
  newconf->useproxy = false;
  newconf->proxyversion = SOCKS4;
  newconf->proxyaddr = "";
  newconf->proxyport = 1080;
  newconf->proxyuser = "";
  newconf->proxypass = "";
  
  newconf->hubsproxytype = NoProxy;
  newconf->hubsproxyaddr = "";
  newconf->hubsproxyport = 80;
  
  /* Find the next free id */
  int id;
  for (id = 0; profiles.find(id); id++);
  profiles.insert(id, newconf);
  return id;
}
    
Config *getConfig(int id)
{
  return profiles.find(id);
}

void killConfig(int id)
{
  profiles.remove(id);
}

QValueList<int> listConfigs()
{
  QValueList<int> configs;
  QIntDictIterator<Config> it(profiles);
  while (it.current()) {
    configs += it.currentKey();
    ++it;
  }
  return configs;
}

int newHub()
{
  DcHubConfig *newconf = new DcHubConfig();
  newconf->name = "New hub";
  newconf->hubaddr = "";
  newconf->nick = "cdget-user";
  newconf->desc = "";
  newconf->email = "nomail@noserver.com";
  newconf->conn = "Cable";
  
  //  newconf->shareddirs; Initialized to empty list...
  newconf->usesizeoffs = false;
  newconf->sizeoffs = 0;
  newconf->usevshare = false;
  newconf->vsharedir = "";
  newconf->maxupl = 3;
  newconf->useullim = false;
  newconf->ullim = 0;

  /* Find the next free id */
  int id;
  for (id = 0; hubconfigs.find(id); id++);
  hubconfigs.insert(id, newconf);
  return id;
}

void killHub(int id)
{
  /* First remove all references to this hub */
  QIntDictIterator<Config> it(profiles);
  for (; it.current(); ++it)
    it.current()->activehubs.remove(id);
  hubconfigs.remove(id);
}

DcHubConfig *getHub(int id)
{
  return hubconfigs.find(id);
}

QValueList<int> listHubs()
{
  QValueList<int> hubs;
  QIntDictIterator<DcHubConfig> it(hubconfigs);
  while (it.current()) {
    hubs += it.currentKey();
    ++it;
  }
  return hubs;
}

void saveTransfer(AudioTransfer trans)
{
  QSettings *s = sw.getSettings();
  s->beginGroup(confprefix);
  s->beginGroup("/running_transfers");
  s->beginGroup("/"+trans.artist+"_"+trans.album+"_"+trans.title);
  s->writeEntry("/artist",trans.artist);
  s->writeEntry("/album",trans.album);
  s->writeEntry("/title",trans.title);
  s->writeEntry("/length",trans.length);
  s->writeEntry("/trackno",trans.trackno);
  s->endGroup();
  s->endGroup();
  s->endGroup();
}

void removeTransfer(AudioTransfer trans)
{
  QSettings *s = sw.getSettings();
  s->beginGroup(confprefix);
  s->beginGroup("/running_transfers");
  s->beginGroup("/"+trans.artist+"_"+trans.album+"_"+trans.title);
  s->removeEntry("/artist");
  s->removeEntry("/album");
  s->removeEntry("/title");
  s->removeEntry("/length");
  s->removeEntry("/trackno");
  s->endGroup();
  s->endGroup();
  s->endGroup();
}

QValueList<AudioTransfer> retrieveTransfers()
{
  QSettings s;
  QValueList<AudioTransfer> tmplst;
  s.beginGroup(confprefix);
  s.beginGroup("/running_transfers");
  QStringList hubs = s.subkeyList("/");
  QStringList::iterator it;

  for ( it = hubs.begin(); it != hubs.end(); ++it) {
    s.beginGroup(*it);
    QString artist = s.readEntry("/artist");
    QString album = s.readEntry("/album");
    QString title = s.readEntry("/title");
    int trackno = s.readNumEntry("/trackno");
    int length = s.readNumEntry("/length");
    tmplst.push_back(AudioTransfer(artist, album, title, trackno, length));
    s.endGroup();
  }
  s.endGroup();
  s.endGroup();
  return tmplst;
}
