#ifndef CONFIGVIEW_H
#define CONFIGVIEW_H

#include "qwidget.h"
#include "qlistview.h"

#include "config.h"
#include "configviewui.h"
#include "hublistgetter.h"

class ConfigView;
class ProfileItem : public QListViewItem
{
public:
  ProfileItem(QListView *parent, ConfigView *confview);
  ProfileItem(QListView *parent, ConfigView *confview, int profile);
  virtual ~ProfileItem();
  bool activateProfile();
  void deactivateProfile();
  bool isActive() { return active; }
private:
  void updateText();
protected:
  virtual void okRename(int col);
  virtual void cancelRename(int col);
  virtual void startRename(int col);
private:
  int profile;
  Config *conf;
  ConfigView *configview;
  bool active;
};

enum HubItemType
{
  AllHubs,ActiveHubs,HubConfigs
};

class HubItem : public QListViewItem
{
public:
  HubItem(QListView *parent, QListViewItem *parent2);
  HubItem(QListView *parent, QListViewItem *parent2, QString name, QString hubaddr);
  HubItem(QListView *parent, QListViewItem *parent2, QString name, QString hubaddr, HubItem *clone);
  HubItem(QListView *parent, QListViewItem *parent2, HubItem *clone);
  HubItem(QListView *parent, QListViewItem *parent2, int hub);
  HubItem(QListView *parent, HubItem *right, bool created = false);
  HubItem(QListViewItem *parent, HubItem *right);
  virtual ~HubItem();

  int getHubId() { return hub; }
  QString getName() { return name; }

  bool isActive() { return active; }
  void activateHub(HubItem *other);
  void deactivateHub();
  void configureHub(ConfigView *confview);
private:
  void rename(QString newname);

protected:
  virtual void okRename(int col);
  virtual void cancelRename(int col);
private:
  HubItemType type;
  int hub;
  QString name;
  HubItem *right;
  HubItem *left;
  bool active;
};


class ConfigView : public ConfigViewUI
{
Q_OBJECT

public:
  ConfigView(QWidget *parent = 0);
  virtual ~ConfigView();
  virtual void show();
  void updateConfigView(const Config &newconf);
  void configureHub(int hub, DcHubConfig *hubconf);
private slots:
  void selectionChanged(QListViewItem *item);
 
  void setViewdetails(); 
  void setDownloaddir();
  void browseDownloaddir();
  void setFileformat();
  void setUseId3v2(bool on);
  void setSetId3(bool on);
  void setRejectId3(bool on);
  void setTempdir();
  void browseTempdir();
  void setTempprefix();
  void setReqArtist(bool on);
  void setReqAlbum(bool on);

  void setMatchchars();
  void setMinbps();
  void setMaxbps();

  void setDelay(int value);
  void setDlcand(int value);
  void setCanddelay();
  void setCandtimeout();
  void setCandtryout();
  void setMaxqueue(int value);
  void setMaxgdlqueue(int value);
  void setNosourcetimeout();
  void setStalledtimeout();
  void setSearchtimeout();
  void setGdlnosourcetmimeout();
  void setGdlstalltimeout();
  void setGdlsourcestalltimeout();
  void setGdlsourceattempts(int value);

  void setHubaddr();
  void setNick();
  void setDesc();
  void setEmail();
  void setConn(const QString &value);
  void setUseSizeoffs();
  void setSizeoffs();
  void setSizeoffsFactor(const QString &f);
  void setUseVshare();
  void setVsharedir();
  void setMaxupl(int value);
  void setUseUllim();
  void setUllim();

  /* Connection options */
  void setActiveMode();
  void setPassiveMode();

  void setUseOverrideIp(bool on);
  void setOverideIp();
  void setUseOverridePort(bool on);
  void setOverridePort();
  void setDynamicIp(bool on);

  void setUseProxy(bool on);
  void setProxyVersion(const QString &version);
  void setProxyAddr();
  void setProxyPort();
  void setProxyUser();
  void setProxyPass();

  void setHubsNoProxy();
  void setHubsWebProxy();
  void setHubsSocksProxy();

  void setHubsProxyAdress();
  void setHubsProxyPort();

  /* Controls */
  void profileActivate();
  void profileNew();
  void profileDelete();

  void hubsAddPub();
  void hubsAddClone();
  void hubsNew();
  void hubsClone();
  void hubsDelete();
  void hubsActivate();
  void hubsDeactivate();
  void hubsApply();

  void sharesAdd();
  void sharesDelete();
private:
  QWidget *current;
  double sizeoffsfactor;
  int selected_hub_id;
  DcHubConfig *selected_hub;
  ProfileItem *lastactive;
  HubListGetter *hubsgetter;
};


#endif // CONFIGVIEW_H
