#include "paintlistviewitem.h"

#include <algorithm>
#include <utility>

#include <qpainter.h>
#include <qregion.h>
#include <qrect.h>

static void destroy_widgets(std::pair<const int, PaintWidget*> &w)
{
    if (w.second != NULL)
        delete w.second;
}

PaintListViewItem::~PaintListViewItem()
{
    std::for_each(widgets.begin(), widgets.end(), destroy_widgets);
}

void PaintListViewItem::addPaintWidget(int column, PaintWidget *w)
{
    std::map<int, PaintWidget*>::const_iterator it = widgets.find(column);
    
    if (it != widgets.end())
    {
        delete (*it).second;
    }

    widgets[column] = w;
}

void PaintListViewItem::paintCell(QPainter *p, const QColorGroup &cg, 
                                  int column, int width, int align)
{
    std::map<int, PaintWidget*>::const_iterator it = widgets.find(column);

    if (it == widgets.end() || (*it).second == NULL)
    {
        QListViewItem::paintCell(p, cg, column, width, align);
    } 
    else
    {
        p->eraseRect(0, 0, width, height());
        PaintWidget *w = (*it).second;
        w->paint(p, cg, height(), width, align);
    }
}

int PaintListViewItem::width(const QFontMetrics &fm, const QListView *lv, 
                             int column) const
{
    std::map<int, PaintWidget*>::const_iterator it = widgets.find(column);

    if (it == widgets.end())
    {
        return QListViewItem::width(fm, lv, column);
    }
    
    PaintWidget *w = (*it).second;

    if (w == NULL)
    {
        return 0;
    }
    else
    {
        return w->width(fm);
    }
}

void PaintListViewItem::paintFocus(QPainter *p, const QColorGroup &cg, 
                                   const QRect &r)
{
    p->setPen(cg.foreground());
    p->drawRect(r);
}

void PaintListViewItem::setText(int column, const QString &text)
{
    std::map<int, PaintWidget*>::const_iterator it = widgets.find(column);

    if (it == widgets.end())
    {
        QListViewItem::setText(column, text);
    } 
    else
    {
        PaintWidget *w = (*it).second;
        
        if (w == NULL)
        {
            QListViewItem::setText(column, text);
        }
        else
        {
            w->setText(text);
        }
    }
}


QString PaintListViewItem::text(int column) const
{
    std::map<int, PaintWidget*>::const_iterator it = widgets.find(column);

    if (it == widgets.end())
    {
        return QListViewItem::text(column);
    } 
    else
    {
        PaintWidget *w = (*it).second;
        
        if (w == NULL)
        {
            return QListViewItem::text(column);
        }
        else
        {
            return w->text();
        }
    }
}

